@@ -189,6 +189,11 @@ if ($db_working) {
 
 // more database tests
 if ($db_working) {
+    // Using transactions to workaround SQLite bug (#7064)
+    if ($DB->db_provider == 'sqlite') {
+        $DB->startTransaction();
+    }
+
     // write test
     $insert_id = md5(uniqid());
     $db_write = $DB->query("INSERT INTO " . $DB->quote_identifier($RCI->config['db_prefix'] . 'session')
@@ -204,6 +209,11 @@ if ($db_working) {
     }
     echo '<br />';
 
+    // Transaction end
+    if ($DB->db_provider == 'sqlite') {
+        $DB->rollbackTransaction();
+    }
+
     // check timezone settings
     $tz_db = 'SELECT ' . $DB->unixtimestamp($DB->now()) . ' AS tz_db';
     $tz_db = $DB->query($tz_db);
