@@ -4,7 +4,7 @@
  * @licstart  The following is the entire license notice for the
  * JavaScript code in this file.
  *
- * Copyright (c) 2006-2014, The Roundcube Dev Team
+ * Copyright (c) The Roundcube Dev Team
  *
  * The JavaScript code in this page is free software: you can redistribute it
  * and/or modify it under the terms of the GNU General Public License
@@ -22,7 +22,7 @@
 function rcube_init_settings_tabs()
 {
   var el, cl, container = $('#tabsbar'),
-    last_tab = $('span:last', container),
+    last_tab = $('span', container).last(),
     tab = '#settingstabpreferences',
     action = window.rcmail && rcmail.env.action ? rcmail.env.action : null;
 
@@ -53,16 +53,19 @@ function rcube_init_tabs(id, current)
 
   current = current ? current : 0;
 
-  // first hide not selected tabs
-  fs.each(function(idx) { if (idx != current) $(this).hide(); });
-
-  // create tabs container
-  var tabs = $('<div>').addClass('tabsbar').appendTo(content);
+  // create tabs container (if not exists)
+  var tabs = content.find('.tabsbar');
+  if (!tabs.length)
+    tabs = $('<div>').addClass('tabsbar').appendTo(content);
 
   // convert fildsets into tabs
   fs.each(function(idx) {
     var tab, a, elm = $(this), legend = elm.children('legend');
 
+    // skip invisible or already initialized fieldsets
+    if (!elm.is(':visible') || elm.hasClass('tabbed'))
+      return;
+
     // create a tab
     a   = $('<a>').text(legend.text()).attr('href', '#');
     tab = $('<span>').attr({'id': 'tab'+idx, 'class': 'tablink'})
@@ -79,6 +82,9 @@ function rcube_init_tabs(id, current)
     // add the tab to container
     tab.append(a).appendTo(tabs);
   });
+
+  // hide not selected tabs
+  fs.each(function(idx) { if (idx != current) $(this).hide(); });
 }
 
 function rcube_show_tab(id, index)
@@ -317,6 +323,11 @@ show_listmenu: function(p)
     $(this).prop('checked', $.inArray(this.value, rcmail.env.listcols) != -1);
   });
 
+  $.each(['widescreen', 'desktop', 'list'], function() {
+    $('input[name="layout"][value="' + this + '"]').prop('checked', rcmail.env.layout == this);
+  });
+  $('#listoptions-columns', $dialog)[rcmail.env.layout == 'widescreen' ? 'hide' : 'show']();
+
   buttons[rcmail.gettext('save')] = function(e) {
     $dialog.dialog('close', e);
     self.save_listmenu();
@@ -354,10 +365,11 @@ save_listmenu: function()
   var sort = $('input[name="sort_col"]:checked').val(),
     ord = $('input[name="sort_ord"]:checked').val(),
     thread = $('input[name="view"]:checked').val(),
+    layout = $('input[name="layout"]:checked').val(),
     cols = $('input[name="list_col[]"]:checked')
       .map(function(){ return this.value; }).get();
 
-  rcmail.set_list_options(cols, sort, ord, thread == 'thread' ? 1 : 0);
+  rcmail.set_list_options(cols, sort, ord, thread == 'thread' ? 1 : 0, layout);
 },
 
 spellmenu: function(show)
@@ -401,12 +413,11 @@ show_attachmentmenu: function(elem, event)
 {
   var id = elem.parentNode.id.replace(/^attach/, '');
 
-  $('#attachmenuopen').off('click').attr('onclick', '').click(function(e) {
-    return rcmail.command('open-attachment', id, this);
-  });
-
-  $('#attachmenudownload').off('click').attr('onclick', '').click(function() {
-    rcmail.command('download-attachment', id, this);
+  $.each(['open', 'download', 'rename'], function() {
+    var action = this;
+    $('#attachmenu' + action).off('click').attr('onclick', '').click(function(e) {
+      return rcmail.command(action + '-attachment', id, this);
+    });
   });
 
   this.popups.attachmentmenu.link = elem;
@@ -457,49 +468,61 @@ body_keydown: function(e)
   }
 },
 
-switch_preview_pane: function(elem)
+// Mail view layout initialization and change handler
+set_layout: function(p)
 {
-  var uid, prev_frm = $('#mailpreviewframe');
+  var layout = p ? p.new_layout : rcmail.env.layout,
+    top = $('#mailcontframe'),
+    bottom = $('#mailpreviewframe');
 
-  if (elem.checked) {
-    rcmail.env.contentframe = 'messagecontframe';
-    if (mailviewsplit.layer) {
-      mailviewsplit.resize();
-      mailviewsplit.layer.elm.style.display = '';
+  if (p)
+    $('#mailrightcontainer').removeClass().addClass(layout);
+
+  if (!this.mailviewsplitv) {
+    this.mailviewsplitv = new rcube_splitter({id:'mailviewsplitterv', p1: 'mailleftcontainer', p2: 'mailrightcontainer',
+      orientation: 'v', relative: true, start: 165, callback: rcube_render_mailboxlist });
+    this.mailviewsplitv.init();
+  }
+
+  $('#mailviewsplitter')[layout == 'desktop' ? 'show' : 'hide']();
+  $('#mailviewsplitter2')[layout == 'widescreen' ? 'show' : 'hide']();
+  $('#mailpreviewframe')[layout != 'list' ? 'show' : 'hide']();
+  rcmail.env.contentframe = layout == 'list' ? null : 'messagecontframe';
+
+  if (layout == 'widescreen') {
+    $('#countcontrols').detach().appendTo($('#messagelistheader'));
+    top.css({height: 'auto', width: 400});
+    bottom.css({top: 0, left: 410, height: 'auto'}).show();
+    if (!this.mailviewsplit2) {
+      this.mailviewsplit2 = new rcube_splitter({id:'mailviewsplitter2', p1: 'mailcontframe', p2: 'mailpreviewframe',
+        orientation: 'v', relative: true, start: 405});
+      this.mailviewsplit2.init();
     }
     else
-      mailviewsplit.init();
-
-    if (bw.opera) {
-      $('#messagelistcontainer').css({height: ''});
-    }
-    prev_frm.show();
-
-    if (uid = rcmail.message_list.get_single_selection())
-      rcmail.show_message(uid, false, true);
+      this.mailviewsplit2.resize();
   }
-  else {
-    prev_frm.hide();
-    if (bw.ie7) {
-      var fr = document.getElementById('mailcontframe');
-      fr.style.bottom = 0;
-      fr.style.height = parseInt(fr.parentNode.offsetHeight)+'px';
+  else if (layout == 'desktop') {
+    top.css({height: 200, width: '100%'});
+    bottom.css({left: 0, top: 210, height: 'auto'}).show();
+    if (!this.mailviewsplit) {
+      this.mailviewsplit = new rcube_splitter({id:'mailviewsplitter', p1: 'mailcontframe', p2: 'mailpreviewframe',
+        orientation: 'h', relative: true, start: 205});
+      this.mailviewsplit.init();
     }
-    else {
-      $('#mailcontframe').css({height: 'auto', bottom: 0});
-      if (bw.opera)
-        $('#messagelistcontainer').css({height: 'auto'});
-    }
-    if (mailviewsplit.layer)
-      mailviewsplit.layer.elm.style.display = 'none';
-
-    rcmail.env.contentframe = null;
-    rcmail.show_contentframe(false);
+    else
+      this.mailviewsplit.resize();
+  }
+  else { // layout == 'list'
+    top.css({height: 'auto', width: '100%'});
+    bottom.hide();
   }
 
-  rcmail.command('save-pref', {name: 'preview_pane', value: (elem.checked?1:0)});
+  if (p && p.old_layout == 'widescreen') {
+    $('#countcontrols').detach().appendTo($('#messagelistfooter'));
+  }
 },
 
+
 /* Message composing */
 init_compose_form: function()
 {
@@ -642,7 +665,11 @@ enable_command: function(p)
   }
   else if (p.command == 'compose-encrypted') {
     // show the toolbar button for Mailvelope
-    $('#messagetoolbar > a.encrypt').show();
+    $('#messagetoolbar a.encrypt').parent().show();
+  }
+  else if (p.command == 'compose-encrypted-signed') {
+    // enable selector for encrypt and sign
+    $('#encryptionmenulink').show();
   }
 },
 
@@ -832,8 +859,8 @@ function rcube_render_mailboxlist()
 {
   var list = $('#mailboxlist > li > a, #mailboxlist ul:visible > li > a');
 
-  // it's too slow with really big number of folders, especially on IE
-  if (list.length > (bw.ie && bw.vendver < 9 ? 40 : 100))
+  // it's too slow with really big number of folders
+  if (list.length > 100)
     return;
 
   list.each(function() {
@@ -984,6 +1011,18 @@ function percent_indicator(obj, data)
   $('#quotaimg').attr('title', data.title);
 };
 
+function attachment_menu_append(item)
+{
+  $(item).append(
+    $('<a class="drop"></a>').on('click keypress', function(e) {
+      if (e.type != 'keypress' || e.which == 13) {
+        rcmail_ui.show_attachmentmenu(this, e);
+        return false;
+      }
+    })
+  );
+};
+
 // Optional parameters used by TinyMCE
 var rcmail_editor_settings = {};
 
@@ -1008,6 +1047,7 @@ function rcube_init_mail_ui()
         .addEventListener('menu-open', 'menu_open', rcmail_ui)
         .addEventListener('aftersend-attachment', 'uploadmenu', rcmail_ui)
         .addEventListener('aftertoggle-editor', 'resize_compose_body_ev', rcmail_ui)
+        .addEventListener('afterbounce', function(){ rcmail_ui.show_popup('forwardmenu', false); })
         .gui_object('dragmenu', 'dragmenu');
 
       if (rcmail.gui_objects.mailboxlist) {
@@ -1027,7 +1067,11 @@ function rcube_init_mail_ui()
           setTimeout(function() { rcmail.message_list.resize(); }, 10);
         });
 
-      if (rcmail.env.action == 'compose') {
+      if (rcmail.env.action == 'list' || !rcmail.env.action) {
+        rcmail.addEventListener('layout-change', 'set_layout', rcmail_ui);
+        rcmail_ui.set_layout();
+      }
+      else if (rcmail.env.action == 'compose') {
         rcmail_ui.init_compose_form();
         rcmail.addEventListener('compose-encrypted', function(e) {
           $("a.button.encrypt")[(e.active ? 'addClass' : 'removeClass')]('selected');
@@ -1035,19 +1079,26 @@ function rcube_init_mail_ui()
           $('a.button.attach, a.button.responses, a.button.attach, #uploadmenulink')[(e.active ? 'addClass' : 'removeClass')]('buttonPas disabled');
           $('#responseslist a.insertresponse')[(e.active ? 'removeClass' : 'addClass')]('active');
         });
+        rcmail.addEventListener('fileappended', function(e) {
+          if (e.attachment.complete)
+            attachment_menu_append(e.item);
+        });
+
+        // add menu link for each attachment
+        $('#attachmentslist > li').each(function() {
+          attachment_menu_append(this);
+        });
       }
       else if (rcmail.env.action == 'show' || rcmail.env.action == 'preview') {
         // add menu link for each attachment
         $('#attachment-list > li[id^="attach"]').each(function() {
-          $(this).append($('<a class="drop"></a>').on('click keypress', function(e) {
-            if (e.type != 'keypress' || e.which == 13) {
-              rcmail_ui.show_attachmentmenu(this, e);
-              return false;
-            }
-          }));
+          attachment_menu_append(this);
         });
 
         $(window).resize(function() {
+          if (!$('#attachment-list > li[id^="attach"]').length)
+            $('#attachment-list').hide();
+
           var mvlpe = $('#messagebody.mailvelope');
           if (mvlpe.length) {
             var content = $('#messageframe'),
@@ -1065,6 +1116,9 @@ function rcube_init_mail_ui()
       if (rcmail.env.action == 'folders') {
         rcmail_ui.folder_search_init($('#folder-manager'));
       }
+
+      $('#mainscreen > #prefs-title').detach().prependTo($('#mainscreen > .box'));
     }
   });
 }
+
